# Print an optspec for argparse to handle cmd's options that are independent of any subcommand.
function __fish_ocm_global_optspecs
	string join \n c/config= p/profile= f/force no-post-action-scripts v/verbose q/quiet h/help V/version
end

function __fish_ocm_needs_command
	# Figure out if the current invocation already has a command.
	set -l cmd (commandline -opc)
	set -e cmd[1]
	argparse -s (__fish_ocm_global_optspecs) -- $cmd 2>/dev/null
	or return
	if set -q argv[1]
		# Also print the command, so this can be used to figure out what it is.
		echo $argv[1]
		return 1
	end
	return 0
end

function __fish_ocm_using_subcommand
	set -l cmd (__fish_ocm_needs_command)
	test -z "$cmd"
	and return 1
	contains -- $cmd[1] $argv
end

complete -c ocm -n "__fish_ocm_needs_command" -s c -l config -d 'Path to the configuration file' -r -F
complete -c ocm -n "__fish_ocm_needs_command" -s p -l profile -d 'Manually select a profile to operate on' -r
complete -c ocm -n "__fish_ocm_needs_command" -s f -l force -d 'Proceed with potentially destructive actions automatically without confirmation'
complete -c ocm -n "__fish_ocm_needs_command" -l no-post-action-scripts -d 'Do not run post-action scripts'
complete -c ocm -n "__fish_ocm_needs_command" -s v -l verbose -d 'Increase logging verbosity'
complete -c ocm -n "__fish_ocm_needs_command" -s q -l quiet -d 'Decrease logging verbosity'
complete -c ocm -n "__fish_ocm_needs_command" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c ocm -n "__fish_ocm_needs_command" -s V -l version -d 'Print version'
complete -c ocm -n "__fish_ocm_needs_command" -f -a "gen" -d 'Generate artefacts like completion scripts and config files'
complete -c ocm -n "__fish_ocm_needs_command" -f -a "profile" -d 'Operations on profiles'
complete -c ocm -n "__fish_ocm_needs_command" -f -a "user" -d 'Operations on users'
complete -c ocm -n "__fish_ocm_needs_command" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and not __fish_seen_subcommand_from completion config help" -s c -l config -d 'Path to the configuration file' -r -F
complete -c ocm -n "__fish_ocm_using_subcommand gen; and not __fish_seen_subcommand_from completion config help" -s p -l profile -d 'Manually select a profile to operate on' -r
complete -c ocm -n "__fish_ocm_using_subcommand gen; and not __fish_seen_subcommand_from completion config help" -s f -l force -d 'Proceed with potentially destructive actions automatically without confirmation'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and not __fish_seen_subcommand_from completion config help" -l no-post-action-scripts -d 'Do not run post-action scripts'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and not __fish_seen_subcommand_from completion config help" -s v -l verbose -d 'Increase logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and not __fish_seen_subcommand_from completion config help" -s q -l quiet -d 'Decrease logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and not __fish_seen_subcommand_from completion config help" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and not __fish_seen_subcommand_from completion config help" -f -a "completion" -d 'Generate shell completion to stdout'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and not __fish_seen_subcommand_from completion config help" -f -a "config" -d 'Initialise a config file'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and not __fish_seen_subcommand_from completion config help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from completion" -s c -l config -d 'Path to the configuration file' -r -F
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from completion" -s p -l profile -d 'Manually select a profile to operate on' -r
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from completion" -s f -l force -d 'Proceed with potentially destructive actions automatically without confirmation'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from completion" -l no-post-action-scripts -d 'Do not run post-action scripts'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from completion" -s v -l verbose -d 'Increase logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from completion" -s q -l quiet -d 'Decrease logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from completion" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from config" -s c -l config -d 'Path to the configuration file' -r -F
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from config" -s p -l profile -d 'Manually select a profile to operate on' -r
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from config" -s f -l force -d 'Proceed with potentially destructive actions automatically without confirmation'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from config" -l no-post-action-scripts -d 'Do not run post-action scripts'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from config" -s v -l verbose -d 'Increase logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from config" -s q -l quiet -d 'Decrease logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from config" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from help" -f -a "completion" -d 'Generate shell completion to stdout'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from help" -f -a "config" -d 'Initialise a config file'
complete -c ocm -n "__fish_ocm_using_subcommand gen; and __fish_seen_subcommand_from help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and not __fish_seen_subcommand_from list help" -s c -l config -d 'Path to the configuration file' -r -F
complete -c ocm -n "__fish_ocm_using_subcommand profile; and not __fish_seen_subcommand_from list help" -s p -l profile -d 'Manually select a profile to operate on' -r
complete -c ocm -n "__fish_ocm_using_subcommand profile; and not __fish_seen_subcommand_from list help" -s f -l force -d 'Proceed with potentially destructive actions automatically without confirmation'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and not __fish_seen_subcommand_from list help" -l no-post-action-scripts -d 'Do not run post-action scripts'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and not __fish_seen_subcommand_from list help" -s v -l verbose -d 'Increase logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and not __fish_seen_subcommand_from list help" -s q -l quiet -d 'Decrease logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and not __fish_seen_subcommand_from list help" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and not __fish_seen_subcommand_from list help" -f -a "list" -d 'List all known profiles'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and not __fish_seen_subcommand_from list help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and __fish_seen_subcommand_from list" -s c -l config -d 'Path to the configuration file' -r -F
complete -c ocm -n "__fish_ocm_using_subcommand profile; and __fish_seen_subcommand_from list" -s p -l profile -d 'Manually select a profile to operate on' -r
complete -c ocm -n "__fish_ocm_using_subcommand profile; and __fish_seen_subcommand_from list" -s f -l force -d 'Proceed with potentially destructive actions automatically without confirmation'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and __fish_seen_subcommand_from list" -l no-post-action-scripts -d 'Do not run post-action scripts'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and __fish_seen_subcommand_from list" -s v -l verbose -d 'Increase logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and __fish_seen_subcommand_from list" -s q -l quiet -d 'Decrease logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and __fish_seen_subcommand_from list" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and __fish_seen_subcommand_from help" -f -a "list" -d 'List all known profiles'
complete -c ocm -n "__fish_ocm_using_subcommand profile; and __fish_seen_subcommand_from help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -s c -l config -d 'Path to the configuration file' -r -F
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -s p -l profile -d 'Manually select a profile to operate on' -r
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -s f -l force -d 'Proceed with potentially destructive actions automatically without confirmation'
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -l no-post-action-scripts -d 'Do not run post-action scripts'
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -s v -l verbose -d 'Increase logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -s q -l quiet -d 'Decrease logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -f -a "list" -d 'List all certificates, with optional filtering'
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -f -a "new" -d 'Generate a certificate for a new user'
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -f -a "rm" -d 'Revoke the certificate for an existing user'
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -f -a "pkg" -d 'Create redistributable packages for the specified users'
complete -c ocm -n "__fish_ocm_using_subcommand user; and not __fish_seen_subcommand_from list new rm pkg help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from list" -s c -l config -d 'Path to the configuration file' -r -F
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from list" -s p -l profile -d 'Manually select a profile to operate on' -r
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from list" -s e -l expired -d 'Only show expired certificates'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from list" -s f -l force -d 'Proceed with potentially destructive actions automatically without confirmation'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from list" -l no-post-action-scripts -d 'Do not run post-action scripts'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from list" -s v -l verbose -d 'Increase logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from list" -s q -l quiet -d 'Decrease logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from list" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from new" -s d -l days -d 'The number of days this certificate stays valid' -r
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from new" -s c -l config -d 'Path to the configuration file' -r -F
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from new" -s p -l profile -d 'Manually select a profile to operate on' -r
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from new" -s f -l force -d 'Proceed with potentially destructive actions automatically without confirmation'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from new" -l no-post-action-scripts -d 'Do not run post-action scripts'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from new" -s v -l verbose -d 'Increase logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from new" -s q -l quiet -d 'Decrease logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from new" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from rm" -s c -l config -d 'Path to the configuration file' -r -F
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from rm" -s p -l profile -d 'Manually select a profile to operate on' -r
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from rm" -l no-update-crl -d 'Do not update crl.pem file'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from rm" -s f -l force -d 'Proceed with potentially destructive actions automatically without confirmation'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from rm" -l no-post-action-scripts -d 'Do not run post-action scripts'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from rm" -s v -l verbose -d 'Increase logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from rm" -s q -l quiet -d 'Decrease logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from rm" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from pkg" -s o -l output-dir -d 'Output to a directory other than the current working directory' -r -f -a "(__fish_complete_directories)"
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from pkg" -s c -l config -d 'Path to the configuration file' -r -F
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from pkg" -s p -l profile -d 'Manually select a profile to operate on' -r
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from pkg" -l add-prefix -l pre -d 'Add the profile name as a prefix to the package name'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from pkg" -l keep-temp -d 'Keep temporary intermediate artifacts instead of deleting them. Helpful for debugging'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from pkg" -s f -l force -d 'Proceed with potentially destructive actions automatically without confirmation'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from pkg" -l no-post-action-scripts -d 'Do not run post-action scripts'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from pkg" -s v -l verbose -d 'Increase logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from pkg" -s q -l quiet -d 'Decrease logging verbosity'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from pkg" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from help" -f -a "list" -d 'List all certificates, with optional filtering'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from help" -f -a "new" -d 'Generate a certificate for a new user'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from help" -f -a "rm" -d 'Revoke the certificate for an existing user'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from help" -f -a "pkg" -d 'Create redistributable packages for the specified users'
complete -c ocm -n "__fish_ocm_using_subcommand user; and __fish_seen_subcommand_from help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c ocm -n "__fish_ocm_using_subcommand help; and not __fish_seen_subcommand_from gen profile user help" -f -a "gen" -d 'Generate artefacts like completion scripts and config files'
complete -c ocm -n "__fish_ocm_using_subcommand help; and not __fish_seen_subcommand_from gen profile user help" -f -a "profile" -d 'Operations on profiles'
complete -c ocm -n "__fish_ocm_using_subcommand help; and not __fish_seen_subcommand_from gen profile user help" -f -a "user" -d 'Operations on users'
complete -c ocm -n "__fish_ocm_using_subcommand help; and not __fish_seen_subcommand_from gen profile user help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c ocm -n "__fish_ocm_using_subcommand help; and __fish_seen_subcommand_from gen" -f -a "completion" -d 'Generate shell completion to stdout'
complete -c ocm -n "__fish_ocm_using_subcommand help; and __fish_seen_subcommand_from gen" -f -a "config" -d 'Initialise a config file'
complete -c ocm -n "__fish_ocm_using_subcommand help; and __fish_seen_subcommand_from profile" -f -a "list" -d 'List all known profiles'
complete -c ocm -n "__fish_ocm_using_subcommand help; and __fish_seen_subcommand_from user" -f -a "list" -d 'List all certificates, with optional filtering'
complete -c ocm -n "__fish_ocm_using_subcommand help; and __fish_seen_subcommand_from user" -f -a "new" -d 'Generate a certificate for a new user'
complete -c ocm -n "__fish_ocm_using_subcommand help; and __fish_seen_subcommand_from user" -f -a "rm" -d 'Revoke the certificate for an existing user'
complete -c ocm -n "__fish_ocm_using_subcommand help; and __fish_seen_subcommand_from user" -f -a "pkg" -d 'Create redistributable packages for the specified users'
